from dune.perftool.generation import get_backend
from dune.perftool.options import option_switch
from dune.perftool.pdelab.argument import (name_applycontainer,
                                           name_coefficientcontainer,
                                           )
from dune.perftool.sumfact.quadrature import (quadrature_inames,
                                              quadrature_weight,
                                              pymbolic_quadrature_position,
                                              )

from dune.perftool.sumfact.basis import (lfs_inames,
                                         pymbolic_basis,
                                         pymbolic_reference_gradient,
                                         pymbolic_coefficient,
                                         pymbolic_coefficient_gradient,
                                         )

import dune.perftool.sumfact.accumulation
import dune.perftool.sumfact.switch

from dune.perftool.pdelab import PDELabInterface


class SumFactInterface(PDELabInterface):
    def lfs_inames(self, element, restriction, number=None, context=''):
        return lfs_inames(element, restriction, number, context)

    def pymbolic_basis(self, element, restriction, number):
        return pymbolic_basis(element, restriction, number)

    def pymbolic_reference_gradient(self, element, restriction, number):
        return pymbolic_reference_gradient(element, restriction, number)

    def pymbolic_trialfunction_gradient(self, element, restriction, component):
        ret, indices = pymbolic_coefficient_gradient(element, restriction, component, name_coefficientcontainer, self.visitor.indices)
        self.visitor.indices = indices
        return ret

    def pymbolic_trialfunction(self, element, restriction, component):
        ret, indices = pymbolic_coefficient(element, restriction, component, name_coefficientcontainer, self.visitor.indices)
        self.visitor.indices = indices
        return ret

    def pymbolic_apply_function_gradient(self, element, restriction, component):
        ret, indices = pymbolic_coefficient_gradient(element, restriction, component, name_applycontainer, self.visitor.indices)
        self.visitor.indices = indices
        return ret

    def pymbolic_apply_function(self, element, restriction, component):
        ret, indices = pymbolic_coefficient(element, restriction, component, name_applycontainer, self.visitor.indices)
        self.visitor.indices = indices
        return ret

    def quadrature_inames(self):
        return quadrature_inames()

    def pymbolic_quadrature_weight(self):
        return quadrature_weight()
#
#    def pymbolic_spatial_coordinate(self):
#        from dune.perftool.pdelab.geometry import to_global
#        return to_global(pymbolic_quadrature_position())

    def pymbolic_spatial_coordinate(self):
        import dune.perftool.sumfact.geometry
        ret, indices = get_backend(interface="spatial_coordinate", selector=option_switch("diagonal_transformation_matrix"))(self.visitor.indices)
        self.visitor.indices = indices
        return ret
